VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ImageSize"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
Private sImagePath As String
Private nImageHeight As Long
Private nImageWidth As Long
Private sErrorMessage As String
Private sImageType As String

Private nFileID As Integer

'States that an image is about to be processed
Public Event Processing(ImagePath As String)

'States that we are done processing the file
Public Event DoneProcessing(ImagePath As String, ImageType As String, Width As Long, Height As Long, ErrorMessage As String)

'Sets the imagepath
Public Property Let ImagePath(sValue As String)
On Error GoTo Error
  sImagePath = sValue
  Call Class_Initialize
  RaiseEvent Processing(sImagePath)
  
  If Not VerifyFile(sImagePath) Then
    sErrorMessage = "This file does not exist!"
    GoTo Error
  End If
  
  nFileID = FreeFile
  Open sImagePath For Binary As #nFileID
  
  sImageType = IdentifyImageType
  If sImageType = "" Then
    sErrorMessage = "This file does not have a valid fileformat!"
    GoTo Error
  End If
  
  Select Case sImageType
    Case "gif"
      Call GetGIFSize
    Case "jpg"
      Call GetJPGSize
    Case "bmp"
      Call GetBMPSize
    Case "png"
      Call GetPNGSize
  End Select
  
Error:
  Close #nFileID
  RaiseEvent DoneProcessing(sImagePath, sImageType, nImageWidth, nImageHeight, sErrorMessage)
End Property

'Returns the imagepath
Public Property Get ImagePath() As String
  ImagePath = sImagePath
End Property

'Returns the imagewidth, else it returns -1 if an error has occured while processing the image
Public Property Get ImageWidth() As Long
  ImageWidth = nImageWidth
End Property

'Returns the imageheight, else it returns -1 if an error has occured while processing the image
Public Property Get ImageHeight() As Long
  ImageHeight = nImageHeight
End Property

'Returns errormessages
Public Property Get ErrorMessage() As String
  ErrorMessage = sErrorMessage
End Property

'Returns the imagetype
'Supported imagetypes are gif, jpg and bmp
Public Property Get ImageType() As String
  ImageType = sImageType
End Property

'Set all defaultvalues
Private Sub Class_Initialize()
  sErrorMessage = ""
  sImageType = ""
  nImageHeight = -1
  nImageWidth = -1
End Sub

'Verifies that a file exists
Private Function VerifyFile(Path) As Boolean
  VerifyFile = False
  If Len(Dir(Path)) Then
    If GetAttr(Path) <> vbDirectory Then
      VerifyFile = True
    End If
  End If
End Function

'Identify imagetype
'Supported imagetypes are gif, jpg and bmp
Private Function IdentifyImageType() As String
On Error GoTo Error
  Dim sId As String, i As Integer
  sId = Space(8)
  Get #nFileID, , sId
  If sId = "GIF" Then
    IdentifyImageType = "gif"
    Exit Function
  End If
  If Asc(Mid(sId, 2, 1)) = 216 And Asc(Mid(sId, 1, 1)) = 255 Then
    IdentifyImageType = "jpg"
    Exit Function
  End If
  If Mid(sId, 1, 2) = "BM" Then
    IdentifyImageType = "bmp"
    Exit Function
  End If
  For i = 1 To 8
    Select Case Asc(Mid(sId, i, 1))
      Case 137, 80, 78, 71, 13, 10, 26, 10
        '
      Case Else
        Exit For
    End Select
  Next i
  If i = 9 Then
    IdentifyImageType = "png"
    Exit Function
  End If
  
Error:
  If Err.Number > 0 Then IdentifyImageType = ""
  Exit Function
End Function

Private Sub GetBMPSize()
  Dim nValue As Long
  'Get headertype
  Get #nFileID, 15, nValue
  If nValue = 12 Then
    Dim nTempWidth As Integer, nTempHeight As Integer
    Get #nFileID, 19, nTempWidth
    Get #nFileID, 21, nTempHeight
    nImageHeight = nTempHeight
    nImageWidth = nTempWidth
  ElseIf nValue >= 40 Then
    Get #nFileID, 19, nImageWidth
    Get #nFileID, 23, nImageHeight
    nImageHeight = Abs(nImageHeight)
    nImageWidth = Abs(nImageWidth)
  Else
    sErrorMessage = "The image is correctly identified as a bmp-file, but the headersize can not be verified"
  End If
End Sub

Private Sub GetGIFSize()
  Dim nTempWidth As Integer, nTempHeight As Integer
  Get #nFileID, 7, nTempWidth
  Get #nFileID, 9, nTempHeight
  nImageHeight = nTempHeight
  nImageWidth = nTempWidth
End Sub

Private Sub GetJPGSize()
  Dim nTempWidth As Integer, nTempHeight As Integer
  Dim nDummy As Byte
  Get #nFileID, 2, nDummy
  
  Do While Not EOF(nFileID)
    nDummy = GetJPGMArker
    If sErrorMessage <> "" Then Exit Do
    Select Case nDummy
      Case 192, 193, 194, 195, 197, 198, 199, 201, 202, 203, 205, 206, 207
        Get #nFileID, , nDummy
        Get #nFileID, , nDummy
        Get #nFileID, , nDummy
        Get #nFileID, , nDummy
        nTempWidth = CLng(nDummy * 256)
        Get #nFileID, , nDummy
        nTempWidth = nTempWidth + CLng(nDummy)
        Get #nFileID, , nDummy
        nTempHeight = CLng(nDummy * 256)
        Get #nFileID, , nDummy
        nTempHeight = nTempHeight + CLng(nDummy)
        nImageHeight = nTempHeight
        nImageWidth = nTempWidth
        Exit Sub
      Case Else
        Get #nFileID, , nDummy
        nTempWidth = CLng(nDummy * 256)
        Get #nFileID, , nDummy
        nTempWidth = nTempWidth + CLng(nDummy)
        For nTempHeight = 3 To nTempWidth
          Get #nFileID, , nDummy
        Next nTempHeight
    End Select
  Loop
  sErrorMessage = "The image is correctly identified as a jpg-file, but the image seems to be corrrupted"
End Sub

Private Function GetJPGMArker() As Byte
  Dim nDiscardedBytes As Long, nDummy As Byte
  Do
    Get #nFileID, , nDummy
    If nDummy <> 255 Then
      nDiscardedBytes = nDiscardedBytes + 1
    Else
      Exit Do
    End If
  Loop
  Do
    If nDummy <> 255 Then Exit Do
    Get #nFileID, , nDummy
  Loop
  If nDiscardedBytes > 0 Then sErrorMessage = "The image is correctly identified as a jpg-file, but the image seems to be corrrupted"
  GetJPGMArker = nDummy
End Function

Private Sub GetPNGSize()
  Dim nDummy As Byte
  Get #nFileID, 17, nDummy
  nImageHeight = nImageHeight + (CLng(nDummy) * (2 ^ 24))
  Get #nFileID, 18, nDummy
  nImageHeight = nImageHeight + (CLng(nDummy) * (2 ^ 16))
  Get #nFileID, 19, nDummy
  nImageHeight = nImageHeight + (CLng(nDummy) * (2 ^ 8))
  Get #nFileID, 20, nDummy
  nImageHeight = nImageHeight + CLng(nDummy) + 1
  Get #nFileID, 21, nDummy
  nImageWidth = nImageWidth + (CLng(nDummy) * (2 ^ 24))
  Get #nFileID, 22, nDummy
  nImageWidth = nImageWidth + (CLng(nDummy) * (2 ^ 16))
  Get #nFileID, 23, nDummy
  nImageWidth = nImageWidth + (CLng(nDummy) * (2 ^ 8))
  Get #nFileID, 24, nDummy
  nImageWidth = nImageWidth + CLng(nDummy) + 1
End Sub
